package kernel;

#if debug
import lib.Debug;
#end
import kernel.service.ServiceManager;
import kernel.fs.FS;
import kernel.gps.GPS;
import kernel.log.Log;
import kernel.net.Routing;
import kernel.ui.WindowManager;
import kernel.net.Net;

class Init {
	@:allow(kernel.KernelEvents)
	public static function initKernel() {
		// Init singeltons here because haxe is confused about the order to create them.
		Log.init();
		KernelEvents.init();

		WindowManager.init();
		MainTerm.instance = new MainTerm();

		Routing.init();
		Net.init();

		GPS.init();

		// Register default terminate handler
		KernelEvents.onTerminate.handle(_ -> {
			KernelEvents.shutdown();
		});

		#if debug
		Debug.printBuildInfo();
		#end

		if (!FS.exists("/var/ns")) {
			FS.makeDir("/var/ns");
		}

		ServiceManager.init();
	}
}
